(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open Conf
open Printf
open Prins

let _ = 
  let alice = {id="alice"; cert="alice"; ip="127.0.0.1"; port=8001} in
  let bob = {id="bob"; cert="bob"; ip="127.0.0.1"; port=8002} in
  let charlie = {id="charlie"; cert="charlie"; ip="127.0.0.1"; port=8003} in
    register alice; register bob; register charlie

let rec handler_decision = { 
    hClose = (fun _ _ -> Done((), "No more revisions")); 
    hReqRevision = (fun _ r -> Revise(r, handler_submission))
}

and handler_submission = { 
    hSubmit = (fun _ submit -> Paper(submit, handler_decision)); 
    hWithdraw = (fun _ _ -> Retract((), "Retracted"))
  }

let rec handler_paper counter prins draft = 
  if counter > 0 then BadFormat("Busy now."^(string_of_int counter),{hUpload = handler_paper (counter-1)}) else 
  if String.length draft > 12 
  then BadFormat("Make it shorter!",{hUpload = handler_paper 0}) 
  else Ok((),handler_submission)

let _ = 
  let result = confman "bob" {hUpload = handler_paper 10} in
  printf "ConfMan: session complete: %s\n\n" result
  
