(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open System.Threading
open Printf

let yield n = System.Threading.Thread.Sleep (n:int)

let fork (c:unit->unit) = 
  let t = new Thread(new ThreadStart(c)) in
  t.Start();
  ()

// local implementation for channels

// Use instead a polymorphic t = System.Collections.Queue?
type 'a chan = Chan of 'a List ref
  
//TODO take a lock!

let chan () = Chan (ref [])
let send (Chan c) v = c := !c @ [v]
  
let recv (Chan c) =  
  let rec f() = match !c with 
  | msg::msgs -> c:= msgs; msg 
  | [] -> yield 1; f () in
  f()
  
// abstract fresh names
type name = Name of string * int

let counter = ref 0 
let name s = 
  incr counter;
  Name(s,!counter) 

let fpname o (Name(s,i)) = Printf.fprintf o "%s%i" s i
let spname (o:unit) (Name(s,i)) = Printf.sprintf "%s%i" s i

// events

type 'a trace = Trace of 'a list ref

let trace () = Trace (ref [])

let log (Trace s) e = s := e::(!s)

let query (Trace s) p = 
  let rec f h = match h with 
  | e::es -> let now = 
      try
        let e' = p e in
        let earlier = List.mem e' es in 
        earlier
      with _ -> true in
      now & f es
  | _ -> true
  in f (!s)

let query2 (Trace s) p = 
  let rec f h = match h with 
  | e::es -> (try let e1,e2 = p e in List.mem e1 es or List.mem e2 es with _ -> true) & f es
  | _ -> true
  in f (!s)

type 'a qfun  = ('a list -> bool)
type 'a query = Query of ('a -> 'a qfun) ref

let new_query () = Query (ref (fun a -> fun b -> true))

let reg_query (Query qu) (f) (s) = 
  let oldq = !qu in
    qu := (fun a -> fun b -> 
	     let r1 = (oldq a b) in
	     let r2 = (f a b) in
	       if r2 then print_string ("Query succeeded: "^s^"\n") 
	       else print_string ("Query failed: "^s^"\n");
	       r1 && r2)

let rec check_query (Query q) (Trace tr)  = 
  match !tr with
      [] -> true
    | h::t -> (!q) h t

let ex (ev:'a) (tr:'a list) = List.mem ev tr
let disj (ev1:'a list -> bool) (ev2:'a list -> bool) (tr:'a list) = (ev1 tr) || (ev2 tr)
let conj (ev1:'a list -> bool) (ev2:'a list -> bool) (tr:'a list) = (ev1 tr) && (ev2 tr)
  
let secret s = name s
